@@ -1,5 +1,5 @@
 # GNU Solfege - ear training for GNOME
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,9 +15,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-MAJOR_VERSION = 3
-MINOR_VERSION = 1
-PATCH_LEVEL = 4
+MAJOR_VERSION = "3"
+MINOR_VERSION = "1"
+PATCH_LEVEL = "5"
 VERSION_STRING = '%s.%s.%s' % (MAJOR_VERSION, MINOR_VERSION, PATCH_LEVEL)
 HAVE_LINUX_AWE_VOICE_H = "yes" == "yes"
 ENABLE_TUNER = "no" == "yes"
