@@ -23,6 +23,7 @@ import sys
 import os
 import cfg
 import gu
+import lessonfilegui
 
 class LearningTree:
     def __init__(self, lessonfile_manager):
@@ -80,6 +81,27 @@ class LearningTree:
         assert topic_idx < len(self.m_topics) - 1
         self.m_topics[topic_idx], self.m_topics[topic_idx + 1] \
             = self.m_topics[topic_idx + 1], self.m_topics[topic_idx]
+    def move_lesson_up(self, topic_idx, lesson_idx):
+        """
+        Return True if successful, else None.
+        The tree is unchanged if we return None.
+        """
+        assert lesson_idx > 0
+        move_id = self.m_topics[topic_idx]['lessons'][lesson_idx]
+        prev_id = self.m_topics[topic_idx]['lessons'][lesson_idx - 1]
+        if prev_id not in list(self.iter_subdeps(move_id)):
+            self.m_topics[topic_idx]['lessons'][lesson_idx-1],\
+              self.m_topics[topic_idx]['lessons'][lesson_idx] = \
+              self.m_topics[topic_idx]['lessons'][lesson_idx],\
+              self.m_topics[topic_idx]['lessons'][lesson_idx-1]
+            return True
+    def move_lesson_down(self, topic_idx, lesson_idx):
+        """
+        Return true if successful, else None.
+        The tree is unchanged if we return None.
+        """
+        assert lesson_idx < len(self.m_topics[topic_idx]['lessons'])
+        return self.move_lesson_up(topic_idx, lesson_idx + 1)
     def set_topic_title(self, topic_idx, new_title):
         self.m_topics[topic_idx]['name'] = new_title
     def add_lesson(self, topic_idx, lesson_id):
@@ -253,6 +275,8 @@ class LessonFileDialogCommon(gtk.Dialog):
             (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,))
         self.m_app = app
         self.m_tree = tree
+        self.m_app.lessonfile_manager.parse()
+        lessonfilegui.handle_lesson_id_crash(self.m_app.lessonfile_manager)
         self.m_cur_topic_idx = cur_topic_idx
         self.g_ok_button = self.add_button("gtk-ok", gtk.RESPONSE_OK)
         self.g_ok_button.set_sensitive(False)
@@ -454,6 +478,15 @@ class Window(gtk.Window):
         self.g_delete_lesson = gtk.Button(_("Delete lesson"))
         self.g_delete_lesson.connect('clicked', self.on_delete_lesson)
         vbox.pack_start(self.g_delete_lesson, False)
+        #
+        self.g_move_lesson_up = gtk.Button(_("Move lesson up"))
+        self.g_move_lesson_up.connect('clicked', self.on_move_lesson_up)
+        vbox.pack_start(self.g_move_lesson_up, False)
+        #
+        self.g_move_lesson_down = gtk.Button(_("Move lesson down"))
+        self.g_move_lesson_down.connect('clicked', self.on_move_lesson_down)
+        vbox.pack_start(self.g_move_lesson_down, False)
+        #
         vbox = gtk.VBox() # The box with the lesson tree and heading
         hbox.pack_start(vbox)
         self.g_lessons_heading = gtk.Label()
@@ -534,7 +567,7 @@ class Window(gtk.Window):
         self.close_window()
     def close_window(self, *v):
         if self.tree_has_changes():
-            if self.m_app.m_ui.dialog_yesno(_("Save changes?")):
+            if gu.dialog_yesno(_("Save changes?"), self):
                 self.on_save()
         self.m_app.m_ui.g_learning_tree_editor = None
         self.destroy()
@@ -619,7 +652,8 @@ class Window(gtk.Window):
         try:
             self.m_tree.save(filename)
         except IOError, e:
-            self.m_app.m_ui.dialog_ok(_("An error occured while saving the file:\n%s") % e)
+            gu.dialog_ok(_("An error occured while saving the file:\n%s") % e,
+                    self)
             return False
         return True
     def on_save(self, *v):
@@ -685,7 +719,7 @@ class Window(gtk.Window):
         if i:
             i = i[0]
             iter = self.g_topics_liststore.iter_nth_child(None, i)
-            if self.m_app.m_ui.dialog_yesno(_("Delete topic '%s'?") % self.m_tree.m_topics[i]['name']):
+            if gu.dialog_yesno(_("Delete topic '%s'?") % self.m_tree.m_topics[i]['name'], self):
                 self.m_tree.delete_topic(i)
                 self.update_gui()
                 self.g_add_lesson.set_sensitive(False)
@@ -722,7 +756,7 @@ class Window(gtk.Window):
             lesson_idx = self.g_lessons.get_cursor()[0][0]
             self.m_tree.delete_dependency(topic_idx, lesson_idx, i)
             self.on_topic_change()
-            #self.on_lesson_change()
+            self.g_lessons.set_cursor("%i" % lesson_idx)
 
     def on_add_dep(self, *v):
         i = self.g_lessons.get_cursor()[0]
@@ -734,10 +768,16 @@ class Window(gtk.Window):
             if ret == gtk.RESPONSE_OK:
                 lesson_id = dlg.get_selected_lesson_id()
                 if lesson_id:
+                    iter = self.g_lessons_liststore.iter_nth_child(None, i)
+                    added_to_id = self.g_lessons_liststore.get(iter, 1)
                     self.m_tree.add_dependency(topic_idx, int(i), lesson_id)
                     self.on_topic_change()
-                    #self.g_topics.set_cursor("%i" % topic_idx)
-                    #self.on_lesson_change()
+                    iter = self.g_lessons_liststore.get_iter_first()
+                    while iter:
+                        if added_to_id == self.g_lessons_liststore.get(iter, 1):
+                            self.g_lessons.set_cursor(self.g_lessons_liststore.get_string_from_iter(iter))
+                            break
+                        iter = self.g_lessons_liststore.iter_next(iter)
             dlg.destroy()
     def on_new_lesson(self, *v):
         topic_idx = self.g_topics.get_cursor()[0][0]
@@ -758,11 +798,35 @@ class Window(gtk.Window):
             do_del = True
             if self.m_tree.get_use_count(del_id) == 1:
                 if self.m_tree.get_dep_use_count(del_id) > 0:
-                    do_del = self.m_app.m_ui.dialog_yesno(_("Lessons depend on this lesson. Delete anyway, and update the dependency list for lessons that depend on this lesson?"))
-                    self.m_tree.remove_all_deps_of(del_id)
+                    do_del = gu.dialog_yesno(_("Lessons depend on this lesson. Delete anyway, and update the dependency list for lessons that depend on this lesson?"), self)
+                    if do_del:
+                        self.m_tree.remove_all_deps_of(del_id)
             if do_del:
                 self.m_tree.delete_lesson(topic_idx, i)
                 self.on_topic_change()
+    def on_move_lesson_up(self, *v):
+        topic_idx = self.g_topics.get_cursor()[0][0]
+        i = self.g_lessons.get_cursor()[0]
+        if i:
+            i = i[0]
+            if i > 0 and self.m_tree.move_lesson_up(topic_idx, i):
+                itera = self.g_lessons_liststore.iter_nth_child(None, i)
+                iterb = self.g_lessons_liststore.iter_nth_child(None, i-1)
+                self.g_lessons_liststore.swap(itera, iterb)
+                self.g_lessons.scroll_to_cell(
+                    self.g_lessons_liststore.get_string_from_iter(itera))
+    def on_move_lesson_down(self, *v):
+        topic_idx = self.g_topics.get_cursor()[0][0]
+        i = self.g_lessons.get_cursor()[0]
+        if i:
+            i = i[0]
+            if i + 1 < len(self.m_tree.m_topics[topic_idx]['lessons']) and \
+                    self.m_tree.move_lesson_down(topic_idx, i):
+                itera = self.g_lessons_liststore.iter_nth_child(None, i)
+                iterb = self.g_lessons_liststore.iter_next(itera)
+                self.g_lessons_liststore.swap(itera, iterb)
+                self.g_lessons.scroll_to_cell(
+                    self.g_lessons_liststore.get_string_from_iter(itera))
     def load_file(self, filename):
         self.m_tree = LearningTree(self.m_app.lessonfile_manager)
         # We use _m_orig_tree just to see if things has changed.
