@@ -100,6 +100,8 @@ predef = {'dictation': 'dictation',
           'twelvetone': 'twelvetone',
           'identifybpm': 'identifybpm',
           #
+          'harmonic': 'harmonic',
+          'melodic': 'melodic',
           'progression': 'progression',
           'normal': 'normal',
           'voice': 'voice',
@@ -852,10 +854,10 @@ class LessonFileManager:
         self.m_uiddb = {}
         vim_tmpfile_re = re.compile("\..*\.swp")
         for collection in cfg.get_list("config/lessoncollections"):
-            if not os.path.isdir(os.path.expanduser(cfg.get_string("lessoncollections/%s" % collection))):
+            if not os.path.isdir(gethomedir.expanduser(cfg.get_string("lessoncollections/%s" % collection))):
                 continue
             # filter out backup files
-            v = filter(lambda x, r=vim_tmpfile_re: x[-1:] != '~' and not r.match(x), os.listdir(os.path.expanduser(cfg.get_string("lessoncollections/%s" % collection))))
+            v = filter(lambda x, r=vim_tmpfile_re: x[-1:] != '~' and not r.match(x), os.listdir(gethomedir.expanduser(cfg.get_string("lessoncollections/%s" % collection))))
             for filename in v:
                 # since I usually run solfege from the source dir:
                 if filename in ('.arch-ids', 'Makefile', 'Makefile.in') \
@@ -869,7 +871,7 @@ class LessonFileManager:
                 # of changes has been done to the lesson file.
                 fn = self.get_complete_filename(collection, filename)
                 h = hash(open(fn, 'r').read())
-                hash_filename = os.path.join(os.path.expanduser("~/.solfege"),
+                hash_filename = os.path.join(gethomedir.expanduser("~/.solfege"),
                            "testresults", "%s_hash" % lesson_id)
                 if os.path.exists(hash_filename):
                     h2 = int(open(hash_filename, 'r').read())
@@ -984,7 +986,7 @@ class LessonFileManager:
         """
         if not catalog:
             catalog = 'solfege'
-        return os.path.expanduser(os.path.join(
+        return gethomedir.expanduser(os.path.join(
          cfg.get_string('lessoncollections/%s' % catalog), fn))
 
 
