@@ -15,6 +15,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+from __future__ import division
 import gtk, gnome.ui, gobject
 import abstract, const, gu, mpd, soundcard
 import random, os
@@ -27,6 +28,17 @@ RHYTHMS = ("c4", "c8 c8", "c16 c16 c16 c16", "c8 c16 c16",
            "r16 c16 c16 c16", "r8 r16 c16", "r16 c8.",
            "c12 c12 c12", "r12 c12 c12",
            "c12 r12 c12", "c12 c12 r12", "r12 r12 c12", "r12 c12 r12",
+           "c4.", "c4 c8", # 22, 23
+           "c8 c4", "c8 c8 c8", # 24, 25
+           "c4 c16 c16", # 26
+           "c16 c16 c4", # 27
+           "c8 c8 c16 c16", #28
+           "c8 c16 c16 c8", #29
+           "c16 c16 c8 c8", #30
+           "c8 c16 c16 c16 c16", #31
+           "c16 c16 c8 c16 c16", #32
+           "c16 c16 c16 c16 c8", #33
+           "c16 c16 c16 c16 c16 c16", #34
            )
 
 
@@ -57,12 +69,11 @@ class Teacher(abstract.LessonbasedTeacher):
 
         norest_v = []
         v = []
-        for x in range(len(RHYTHMS)):
-            if self.get_bool("rhythm_element_%i" % x):
-                if not (RHYTHMS[x][0] == "r"
-                        and self.get_bool("not_start_with_rest")):
-                    norest_v.append(x)
-                v.append(x)
+        for x in self.m_P.header.rhythm_elements:
+            if not (RHYTHMS[x][0] == "r"
+                    and self.get_bool("not_start_with_rest")):
+                norest_v.append(x)
+            v.append(x)
         if not v:
             return Teacher.ERR_NO_ELEMS
         if not norest_v:
@@ -73,9 +84,13 @@ class Teacher(abstract.LessonbasedTeacher):
         self.q_status = const.QSTATUS_NEW
         return Teacher.OK
     def get_music_string(self):
+        if self.m_P.header.count_in_notelen:
+            count_in_notelen = self.m_P.header.count_in_notelen
+        else:
+            count_in_notelen = "4"
         s = ""
         for x in range(self.get_int("count_in")):
-            s = s + "d4 "
+            s = s + "d%s " % count_in_notelen
         for k in self.m_question:
             s = s + RHYTHMS[k] + " "
         return r"\staff{%s}" % s
@@ -119,8 +134,7 @@ def create_rhythm_image(rhythm):
     The image returned is shown.
     """
     im = gtk.Image()
-    im.set_from_stock('solfege-rhythm-%s' % ''.join(rhythm.split()),
-                      gtk.ICON_SIZE_LARGE_TOOLBAR)
+    im.set_from_file(os.path.join('graphics', 'rhythm-%s.png' % (rhythm.replace(" ", ""))))
     im.show()
     return im
 
@@ -246,21 +260,11 @@ class RhythmViewer(gtk.Frame):
 
 
 class Gui(abstract.Gui):
+    num_buttons_in_row = 8
     def __init__(self, teacher, window):
         abstract.Gui.__init__(self, teacher, window)
         self.m_key_bindings = {'backspace_ak': self.on_backspace}
-        hbox = gu.bHBox(self.practise_box, False)
-        for i in range(0, 8):
-            b = self.pngbutton(i)
-            if i == 0:
-                self.g_first_rhythm_button = b
-            hbox.pack_start(b, False)
-        hbox = gu.bHBox(self.practise_box, False)
-        for i in range(8, 16):
-            hbox.pack_start(self.pngbutton(i), False)
-        hbox = gu.bHBox(self.practise_box, False)
-        for i in range(16, 22):
-            hbox.pack_start(self.pngbutton(i), False)
+        self.g_answer_box = gu.bVBox(self.practise_box, False)
         #-------
         self.practise_box.pack_start(gtk.HBox(), False,
                                      padding=gnome.ui.PAD_SMALL)
@@ -289,15 +293,15 @@ class Gui(abstract.Gui):
         vbox = gtk.VBox()
         vbox.set_border_width(gnome.ui.PAD_SMALL)
         frame.add(vbox)
-        hbox = gu.bHBox(vbox, False)
-        for i in range(0, 8):
-            hbox.pack_start(self.pngcheckbutton(i), False)
-        hbox = gu.bHBox(vbox, False)
-        for i in range(8, 16):
-            hbox.pack_start(self.pngcheckbutton(i), False)
-        hbox = gu.bHBox(vbox, False)
-        for i in range(16, 22):
-            hbox.pack_start(self.pngcheckbutton(i), False)
+        sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
+        self.m_element_check_buttons = {}
+        for i in range(len(RHYTHMS)):
+            if i % self.num_buttons_in_row == 0:
+                hbox = gu.bHBox(vbox, False)
+            b = self.pngcheckbutton(i)
+            self.m_element_check_buttons[i] = b
+            sizegroup.add_widget(b)
+            hbox.pack_start(b, False)
         #--------    
         self.config_box.pack_start(gtk.HBox(), False,
                                    padding=gnome.ui.PAD_SMALL)
@@ -343,15 +347,32 @@ class Gui(abstract.Gui):
     def pngbutton(self, i):
         "used by the constructor"
         btn = gtk.Button()
-        btn.add(create_rhythm_image(RHYTHMS[i]))
+        if i > len(RHYTHMS):
+            im = gtk.Image()
+            im.set_from_stock("gtk-missing-image", gtk.ICON_SIZE_LARGE_TOOLBAR)
+            im.show()
+            btn.add(im)
+        else:
+            btn.add(create_rhythm_image(RHYTHMS[i]))
         btn.show()
         btn.connect('clicked', self.guess_element, i)
         return btn
     def pngcheckbutton(self, i):
-        btn = gu.nCheckButton(self.m_exname, "rhythm_element_%i" % i, default_value=1)
+        btn = gtk.CheckButton()
         btn.add(create_rhythm_image(RHYTHMS[i]))
         btn.show()
+        btn.connect('clicked', self.select_element_cb, i)
         return btn
+    def select_element_cb(self, button, element_num):
+        if button.get_active():
+            if element_num not in self.m_t.m_P.header.rhythm_elements:
+                self.m_t.m_P.header.rhythm_elements.append(element_num)
+                self.m_t.m_P.header.visible_rhythm_elements.append(element_num)
+        else:
+            if element_num in self.m_t.m_P.header.rhythm_elements:
+                self.m_t.m_P.header.rhythm_elements.remove(element_num)
+                self.m_t.m_P.header.visible_rhythm_elements.remove(element_num)
+        self.update_answer_buttons()
     def on_backspace(self, widget=None):
         if self.m_t.q_status == const.QSTATUS_SOLVED:
             return
@@ -404,13 +425,40 @@ class Gui(abstract.Gui):
     def repeat_question(self, *w):
         self.m_t.play_question()
         self.g_first_rhythm_button.grab_focus()
+    def update_select_elements_buttons(self):
+        for n in range(len(RHYTHMS)):
+            self.m_element_check_buttons[n].set_active(
+                n in self.m_t.m_P.header.rhythm_elements)
+    def update_answer_buttons(self):
+        self.g_answer_box.foreach(lambda w: w.destroy())
+        w = self.num_buttons_in_row
+        c = 0
+        visible_rhythm_elements = self.m_t.m_P.header.visible_rhythm_elements
+        if not visible_rhythm_elements:
+            visible_rhythm_elements = self.m_t.m_P.header.rhythm_elements
+        num_lines = len(visible_rhythm_elements) // w + 1
+        # adjust the width, so that we don't get one last almost empty line
+        w = len(visible_rhythm_elements) // num_lines + 1
+        sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
+        for n in visible_rhythm_elements:
+            if c % w == 0:
+                hbox = gu.bHBox(self.g_answer_box, True)
+            b = self.pngbutton(n)
+            sizegroup.add_widget(b)
+            if c == 0:
+                self.g_first_rhythm_button = b
+            hbox.pack_start(b, False)
+            c += 1
     def on_start_practise(self):
-        if self.m_t.m_P.header.rhythm_elements:
-            for n in range(22):
-                self.m_t.set_bool('rhythm_element_%i' % n, 
-                              n in self.m_t.m_P.header.rhythm_elements)
-        self.m_t.m_custom_mode = bool(
-            not self.m_t.m_P.header.rhythm_elements)
+        assert self.m_t.m_P
+        if not self.m_t.m_P.header.rhythm_elements:
+            self.m_t.m_custom_mode = True
+            self.m_t.m_P.header.rhythm_elements = [0, 1, 2]
+            self.m_t.m_P.header.visible_rhythm_elements = [0, 1, 2]
+        else:
+            self.m_t.m_custom_mode = False
+        self.update_select_elements_buttons()
+        self.update_answer_buttons()
         if self.m_t.m_custom_mode:
             self.g_element_frame.show()
         else:
@@ -419,11 +467,15 @@ class Gui(abstract.Gui):
             self.set_lesson_heading(self.m_t.m_P.header.lesson_heading)
         else:
             self.set_lesson_heading(_("Identify the rhythm"))
-        for n in ('bpm',
-                  'countin_perc', 'count_in',
-                  'rhythm_perc', 'num_beats'):
+        for n, default in (('bpm', 60),
+                  ('countin_perc', 80),
+                  ('count_in', 2),
+                  ('rhythm_perc', 37),
+                  ('num_beats', 4)):
             if n in self.m_t.m_P.header:
                 self.m_t.set_int(n, self.m_t.m_P.header[n])
+            else:
+                self.m_t.set_int(n, default)
         if 'not_start_with_rest' in self.m_t.m_P.header:
             self.m_t.set_bool('not_start_with_rest', self.m_t.m_P.header.not_start_with_rest)
         self.g_rhythm_viewer.flash(_("Click 'New' to begin."))
